/* 
** $Header: /u/drspeech/repos/rtst/rtst.h,v 1.43 1996/11/25 20:43:31 krste Exp $
*/

/***************************************************************
** rtst - various routines useful for writing test programs.
**
** They can be divided into several categories:
**
**   Datatypes and constants used by the library.
**   Test control functions.
**   Test result checking.
**   Random test data generation.
**
***************************************************************/

#ifndef rtst_h_INCLUDED
#define rtst_h_INCLUDED

#include <stddef.h>
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************
** Datatypes and constants used by the library.
***************************************************************/

typedef signed char rtst_int8;
typedef unsigned char rtst_uint8;
typedef short rtst_int16;
typedef unsigned short rtst_uint16;
typedef int rtst_int32;
typedef unsigned int rtst_uint32;
typedef int rtst_bool;

#define RTST_INT8_MAX (0x7f)
#define RTST_INT8_MIN (-0x80)
#define RTST_INT16_MAX (0x7fff)
#define RTST_INT16_MIN (-0x8000)
#define RTST_INT32_MAX (0x7fffffff)
#define RTST_INT32_MIN (-0x7fffffff-1)

#define RTST_UINT8_MAX (0xffu)
#define RTST_UINT16_MAX (0xffffu)
#define RTST_UINT32_MAX (0xffffffffu)

#define RTST_TRUE (1)
#define RTST_FALSE (0)

/***************************************************************
** Routines used internally to generate NaNs and Infs.
***************************************************************/

extern float rtst_ieee_divide_ff_f(float num, float denom);
extern double rtst_ieee_divide_dd_d(double num, double denom);

/***************************************************************
** Test control.
**
** Control is just the encapsulation of some simple functionality involved in
** many test programs.  This includes handling the start and end of tests and
** producing output.
**
** Specifically, we need to be able to:
**  - optionally output status messages
**  - seed random number generators
**  - delimit the start and end of the test
**  - have control over the time a random test is run
**  - have control over the size of random tests
**
** This stuff needs to be controlled by the command line, hence we have
** a command line parsing routine.  This command line parsing routine must
** be called before any use of stdout.
***************************************************************/

/* This is the default number of iterations for random tests. */
#define RTST_DEFAULT_NUMTESTS (100)

/* This is the default size of random tests. */
#define RTST_DEFAULT_SIZETESTS (100)

/*
** This specifies the stream to send logging output to - normally stdout
** (no logging if NULL).  This logging output is line buffered with a call
** to setvbuf(), so the rtst_args() routine must be called before any writes to
** stdout. 
*/

extern FILE* rtst_logfile;

/*
** This is the random number generator state used by the rtst package.
** rtst uses the rand48 family of generators.
*/
extern unsigned short rtst_state48[3];

/* This is the name of the current test */
extern const char* rtst_name;

/*
** This variable is used to control the number of random tests performed,
** although the number of tests need only be proportional to this.
** This parameter set with -n <number> on the command line.
*/
extern int rtst_numtests;

/*
** This variable is used to control the size of random tests performed,
** although the actual size parameters need only be proportional to this.
** This parameter set with -s <number> on the command line.
*/
extern size_t rtst_sizetests;

/*
  Read command line arguments.
  Arguments recognized here are:

  -v                  turn on verbose logging.
  -t                  use output of time() to set seed value
  -r <seed>           use given value to seed random number generator
  -n <number>         set rtst_numtests.
  -s <size>           set rtst_sizetests.

  Returns remaining argc and argv.
  There is a fixed default seed.
*/
extern int
rtst_args(int argc, char* argv[]);

/*
** Display a message if logging is enabled
*/
extern void
rtst_log(const char* string, ...);

/* Indicate the start of a specific test */
extern void
rtst_start(const char* name);

/*
** Indicate successful completion of the test.
** If the verbose option is specified on the
** command line, it will also output a message.
*/
extern void
rtst_passed(void);

/*
** Indicate unsuccessful completion of the current test.
** The arguments are handled
** as in printf, and output is produced regardless of the verbose option.
** Exit is called with a non-zero value to terminate the program.
*/
extern void
rtst_failed(const char* format, ...);

/*
** Exit from the test program - returns a "success" exit code
*/
extern void
rtst_exit(void);

/*****************************************************************************
** Random test data generation.
**
** These routines generate random scalars and vectors of various types, and
** with various distributions.
**
** All of these routines use the standard rand48 generator, which is seeded
** from the command line.
******************************************************************************/

/*****************************************************************************
** These urand functions provide uniform distributions over a range specified
** by the min and max arguments.
** The first set of functions return a single value, while the second set fill
** a vector with random values.
******************************************************************************/

extern rtst_int8
rtst_urand_i8i8_i8(rtst_int8 min, rtst_int8 max);

extern rtst_uint8
rtst_urand_u8u8_u8(rtst_uint8 min, rtst_uint8 max);

extern rtst_int16
rtst_urand_i16i16_i16(rtst_int16 min, rtst_int16 max);

extern rtst_uint16
rtst_urand_u16u16_u16(rtst_uint16 min, rtst_uint16 max);

extern rtst_int32
rtst_urand_i32i32_i32(rtst_int32 min, rtst_int32 max);

extern rtst_uint32
rtst_urand_u32u32_u32(rtst_uint32 min, rtst_uint32 max);

#define rtst_urand_ff_f(min, max) ((float) rtst_urand_dd_d((min), (max)))

extern double
rtst_urand_dd_d(double min, double max);

extern void
rtst_urand_i8i8_vi8(size_t n, rtst_int8 min, rtst_int8 max, rtst_int8* vec);

extern void
rtst_urand_u8u8_vu8(size_t n, rtst_uint8 min, rtst_uint8 max, rtst_uint8* vec);

extern void
rtst_urand_i16i16_vi16(size_t n, rtst_int16 min, rtst_int16 max,
                       rtst_int16* vec);

extern void
rtst_urand_u16u16_vu16(size_t n, rtst_uint16 min, rtst_uint16 max,
                       rtst_uint16* vec);

extern void
rtst_urand_i32i32_vi32(size_t n, rtst_int32 min, rtst_int32 max,
                       rtst_int32* vec);

extern void
rtst_urand_u32u32_vu32(size_t n, rtst_uint32 min, rtst_uint32 max,
                       rtst_uint32* vec);

extern void
rtst_urand_ff_vf(size_t n, float min, float max, float* vec);

extern void
rtst_urand_dd_vd(size_t n, double min, double max, double* vec);

/*
  Floating point arithmetic is not associatve and hence two routines that
  perform operations in a slightly different order can give different
  results.  One approach that checks the structure of a computation is to
  use small integers in the floating point vectors to provide a
  "signature".  Multiplies and adds with integers are exact under IEEE
  arithmetic and so two routines using only these operations should give
  exactly identical results.

  These urandi functions fill a floating point vector with uniform
  distributions of integers over a range specified by the min and max
  arguments.
*/

extern void
rtst_urandi_i32i32_vf(size_t n, rtst_int32 min, rtst_int32 max, float *vec);

extern void
rtst_urandi_i32i32_vd(size_t n, rtst_int32 min, rtst_int32 max, double *vec);

/*
** Just using uniform random numbers as test data is often
** inappropriate - here we have various routines for producing biased
** test data.
** 
** The routines generating "bad" random numbers are useful.  "bad" depends
** on what type of number you're producing, but might e.g. include
** lots of 0s, maximums, minimums, one bit low etc.
**
** The first set of functions produce a single value, while the second set
** produce a vector of values.
*/

extern rtst_int16
rtst_badrand_i16(void);

extern rtst_int32
rtst_badrand_i32(void);

extern void
rtst_badrand_vi16(size_t len, rtst_int16* vec);

extern void
rtst_badrand_vi32(size_t len, rtst_int32* vec);

/***************************************************************
** Checking results.
**
** These macros are used to check results of tests.
** On failure, they indicate the offending file, line and index
** and exit the test.
***************************************************************/

/*
** An assertion macro. This is just like normal assert except that it makes
** the output noticeable by using rtst_failed, always tests regardless of
** NDEBUG, and exits rather than aborting.
*/

#define rtst_assert(ex) \
  (void)((ex) || (rtst_failed("%s:%d: rtst_assert failed.", __FILE__, __LINE__),0))

/*
** These macros test vectors of values for equality.  There are several
** versions depending on the type of value in the vector.
*/

#define rtst_checkeq_vi8vi8(len, a, b) \
    _rtst_checkeq_vi8vi8(len, a, b, __FILE__, __LINE__)

#define rtst_checkeq_vu8vu8(len, a, b) \
    _rtst_checkeq_vu8vu8(len, a, b, __FILE__, __LINE__)

#define rtst_checkeq_vi16vi16(len, a, b) \
    _rtst_checkeq_vi16vi16(len, a, b, __FILE__, __LINE__)

#define rtst_checkeq_vu16vu16(len, a, b) \
    _rtst_checkeq_vu16vu16(len, a, b, __FILE__, __LINE__)

#define rtst_checkeq_vi32vi32(len, a, b) \
    _rtst_checkeq_vi32vi32(len, a, b, __FILE__, __LINE__)

#define rtst_checkeq_vu32vu32(len, a, b) \
    _rtst_checkeq_vu32vu32(len, a, b, __FILE__, __LINE__)

#define rtst_checkeq_vfvf(len, a, b) \
    _rtst_checkeq_vfvf(len, a, b, __FILE__, __LINE__)

#define rtst_checkeq_vdvd(len, a, b) \
    _rtst_checkeq_vdvd(len, a, b, __FILE__, __LINE__)

extern void _rtst_checkeq_vi8vi8(size_t len, const rtst_int8* a,
                                   const rtst_int8* b,
                                   const char* file, unsigned int line);

extern void _rtst_checkeq_vu8vu8(size_t len, const rtst_uint8* a,
                                   const rtst_uint8* b,
                                   const char* file, unsigned int line);

extern void _rtst_checkeq_vi16vi16(size_t len, const rtst_int16* a,
                                   const rtst_int16* b,
                                   const char* file, unsigned int line);

extern void _rtst_checkeq_vu16vu16(size_t len, const rtst_uint16* a,
                                   const rtst_uint16* b,
                                   const char* file, unsigned int line);

extern void _rtst_checkeq_vi32vi32(size_t len, const rtst_int32* a,
                                   const rtst_int32* b,
                                   const char* file, unsigned int line);

extern void _rtst_checkeq_vu32vu32(size_t len, const rtst_uint32* a,
                                   const rtst_uint32* b,
                                   const char* file, unsigned int line);

extern void _rtst_checkeq_vfvf(size_t len, const float* a, const float* b,
                               const char* file, unsigned int line);

extern void _rtst_checkeq_vdvd(size_t len, const double* a, const double* b,
                               const char* file, unsigned int line);

/*
** These macros test matrices of values for equality.  The
** matrices must have the same number of rows or columns, but can have
** differing LDAs.
*/

#define rtst_checkeq_mi8mi8(rows, cols, lda_a, lda_b, a, b) \
    _rtst_checkeq_mi8mi8(rows, cols, lda_a, lda_b, a, b, __FILE__, __LINE__)
#define rtst_checkeq_mi16mi16(rows, cols, lda_a, lda_b, a, b) \
    _rtst_checkeq_mi16mi16(rows, cols, lda_a, lda_b, a, b, __FILE__, __LINE__)
#define rtst_checkeq_mi32mi32(rows, cols, lda_a, lda_b, a, b) \
    _rtst_checkeq_mi32mi32(rows, cols, lda_a, lda_b, a, b, __FILE__, __LINE__)
#define rtst_checkeq_mfmf(rows, cols, lda_a, lda_b, a, b) \
    _rtst_checkeq_mfmf(rows, cols, lda_a, lda_b, a, b, __FILE__, __LINE__)
#define rtst_checkeq_mdmd(rows, cols, lda_a, lda_b, a, b) \
    _rtst_checkeq_mdmd(rows, cols, lda_a, lda_b, a, b, __FILE__, __LINE__)

extern void
_rtst_checkeq_mi8mi8(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const rtst_int8* a, const rtst_int8* b,
		   const char* file, unsigned int line);

extern void
_rtst_checkeq_mi16mi16(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const rtst_int16* a, const rtst_int16* b,
		   const char* file, unsigned int line);

extern void
_rtst_checkeq_mi32mi32(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const rtst_int32* a, const rtst_int32* b,
		   const char* file, unsigned int line);

extern void
_rtst_checkeq_mfmf(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const float* a, const float* b,
		   const char* file, unsigned int line);

extern void
_rtst_checkeq_mdmd(size_t rows, size_t cols, size_t a_lda, size_t b_lda,
		   const double* a, const double* b,
		   const char* file, unsigned int line);
/*
** These macros test for two vectors being the same within a given
** tolerance.
*/

#define rtst_checknear_fvfvf(len, tolerance, a, b) \
    _rtst_checknear_fvfvf(len, tolerance, a, b, __FILE__, __LINE__)

#define rtst_checknear_dvdvd(len, tolerance, a, b) \
    _rtst_checknear_dvdvd(len, tolerance, a, b, __FILE__, __LINE__)

extern void _rtst_checknear_fvfvf(size_t len, float tolerance,
				  const float* a, const float* b,
				  const char* file, unsigned int line);

extern void _rtst_checknear_dvdvd(size_t len, double tolerance,
				  const double* a, const double* b,
				  const char* file, unsigned int line);

/*
** These macros test for a vector being within a given range
*/

#define rtst_checkrange_i16i16vi16(len, min, max, in) \
    _rtst_checkrange_i16i16vi16(len, min, max, in, __FILE__, __LINE__)

#define rtst_checkrange_i32i32vi32(len, min, max, in) \
    _rtst_checkrange_i32i32vi32(len, min, max, in, __FILE__, __LINE__)

#define rtst_checkrange_ffvf(len, min, max, in) \
    _rtst_checkrange_ffvf(len, min, max, in, __FILE__, __LINE__)

#define rtst_checkrange_ddvd(len, min, max, in) \
    _rtst_checkrange_ddvd(len, min, max, in, __FILE__, __LINE__)

extern void
_rtst_checkrange_i16i16vi16(size_t len, rtst_int16 min, rtst_int16 max,
			    const rtst_int16* in,
			    const char* file, unsigned int line);

extern void
_rtst_checkrange_i32i32vi32(size_t len, rtst_int32 min, rtst_int32 max,
			    const rtst_int32* in,
			    const char* file, unsigned int line);

extern void
_rtst_checkrange_ffvf(size_t len, float min, float max, const float* in,
		      const char* file, unsigned int line);

extern void
_rtst_checkrange_ddvd(size_t len, double min, double max, double * in,
		      const char* file, unsigned int line);

/*****************************************************************************

  Padvecs are allocated like normal vectors except that the padvec library
  maintains extra invisible padding at each end of the vector, and the vector
  can be aligned at any byte bounday that is legal for the vector type in
  question - not just the most restrictive as with normal malloc.

  The padding is initialized with a random signature that the padvec library
  can check for corruption. The check is always performed when a padvec is
  freed.

*****************************************************************************/

/* Create routines. */

/* Allocate a vector of length n_bytes, with alignment of at least */
/* align_bytes. Align_bytes must be a power of 2. */
extern void *
rtst_padvec_new(size_t n_bytes, int align_bytes);

/* Macros for various types. */

#define rtst_padvec_new_vi8(len) \
   ((rtst_int8 *) rtst_padvec_new((len)*sizeof(rtst_int8), sizeof(rtst_int8)))

#define rtst_padvec_new_vu8(len) \
   ((rtst_uint8 *) rtst_padvec_new((len)*sizeof(rtst_uint8), \
                                   sizeof(rtst_uint8)))

#define rtst_padvec_new_vi16(len) \
   ((rtst_int16 *) rtst_padvec_new((len)*sizeof(rtst_int16), \
                                    sizeof(rtst_int16)))

#define rtst_padvec_new_vu16(len) \
   ((rtst_uint16 *) rtst_padvec_new((len)*sizeof(rtst_uint16), \
                                    sizeof(rtst_uint16)))

#define rtst_padvec_new_vi32(len) \
   ((rtst_int32 *) rtst_padvec_new((len)*sizeof(rtst_int32), \
                                    sizeof(rtst_int32)))

#define rtst_padvec_new_vu32(len) \
   ((rtst_uint32 *) rtst_padvec_new((len)*sizeof(rtst_uint32), \
                                    sizeof(rtst_uint32)))

#define rtst_padvec_new_vf(len) \
   ((float *) rtst_padvec_new((len)*sizeof(float), sizeof(float)))

#define rtst_padvec_new_vd(len) \
   ((double *) rtst_padvec_new((len)*sizeof(double), sizeof(double)))

/* Create and copy routines.  These copy the argument vector into the */
/* allocated space. */

extern void *
rtst_padvec_clone(size_t n_bytes, int align_bytes, void* input);

#define rtst_padvec_clone_vi8(len, vec) \
    ((rtst_int8 *) rtst_padvec_clone((len)*sizeof(rtst_int8), \
                                     sizeof(rtst_int8), (void *) vec))

#define rtst_padvec_clone_vu8(len, vec) \
    ((rtst_uint8 *) rtst_padvec_clone((len)*sizeof(rtst_uint8), \
                                     sizeof(rtst_uint8), (void *) vec))

#define rtst_padvec_clone_vi16(len, vec) \
    ((rtst_int16 *) rtst_padvec_clone((len)*sizeof(rtst_int16), \
                                      sizeof(rtst_int16), (void *) vec))

#define rtst_padvec_clone_vu16(len, vec) \
    ((rtst_uint16 *) rtst_padvec_clone((len)*sizeof(rtst_uint16), \
                                      sizeof(rtst_uint16), (void *) vec))

#define rtst_padvec_clone_vi32(len, vec) \
    ((rtst_int32 *) rtst_padvec_clone((len)*sizeof(rtst_int32), \
                                      sizeof(rtst_int32), (void *) vec))

#define rtst_padvec_clone_vu32(len, vec) \
    ((rtst_uint32 *) rtst_padvec_clone((len)*sizeof(rtst_uint32), \
                                      sizeof(rtst_uint32), (void *) vec))

#define rtst_padvec_clone_vf(len, vec) \
    ((float *) rtst_padvec_clone((len)*sizeof(float), \
                                 sizeof(float), (void *) vec))

#define rtst_padvec_clone_vd(len, vec) \
    ((double *) rtst_padvec_clone((len)*sizeof(double), \
                                  sizeof(double), (void *) vec))

/* Check routines. */

extern void
_rtst_padvec_check(void *p, const char *file, unsigned int line);

#define rtst_padvec_check_vi8(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_check_vu8(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_check_vi16(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_check_vu16(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_check_vi32(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_check_vu32(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_check_vf(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_check_vd(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_check(p) \
    _rtst_padvec_check((void *)(p), __FILE__, __LINE__)

/* Delete routines. */

extern void
_rtst_padvec_del(void *p, const char *file, unsigned int line);

#define rtst_padvec_del_vi8(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_del_vu8(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_del_vi16(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_del_vu16(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_del_vi32(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_del_vu32(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_del_vf(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_del_vd(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

#define rtst_padvec_del(p) \
    _rtst_padvec_del((void *)(p), __FILE__, __LINE__)

/*****************************************************************************

  Padmats are to C matrices as padvecs are to vectors.  The only substantial
  difference is that the "unused" part at the end of each row is also
  padded and checked.

*****************************************************************************/

/* Create routines. */

/* Allocate a matrix of bytes size rows*lda_bytes, with the last */
/* (lda_bytes-col_bytes) of each row padded.  Aligned to */
/* align_bytes. Align_bytes must be a power of 2. */
extern void *
rtst_padmat_new(size_t rows, size_t col_bytes, size_t lda_bytes,
		int align_bytes);

/* Macros for various types. */

#define rtst_padmat_new_mi8(rows, cols, lda) \
   ((rtst_int8 *) rtst_padmat_new(rows, (cols)*sizeof(rtst_int8), \
			      (lda)*sizeof(rtst_int8), sizeof(rtst_int8)))

#define rtst_padmat_new_mi16(rows, cols, lda) \
   ((rtst_int16 *) rtst_padmat_new(rows, (cols)*sizeof(rtst_int16), \
			      (lda)*sizeof(rtst_int16), sizeof(rtst_int16)))

#define rtst_padmat_new_mi32(rows, cols, lda) \
   ((rtst_int32 *) rtst_padmat_new(rows, (cols)*sizeof(rtst_int32), \
			      (lda)*sizeof(rtst_int32), sizeof(rtst_int32)))

#define rtst_padmat_new_mf(rows, cols, lda) \
   ((float *) rtst_padmat_new(rows, (cols)*sizeof(float), \
			      (lda)*sizeof(float), sizeof(float)))

#define rtst_padmat_new_md(rows, cols, lda) \
   ((double *) rtst_padmat_new(rows, (cols)*sizeof(double), \
			      (lda)*sizeof(double), sizeof(double)))

/* Create and copy routines.  These copy the argument vector into the */
/* allocated space. */

extern void *
rtst_padmat_clone(size_t rows, size_t col_bytes, size_t in_lda_bytes,
		  size_t res_lda_bytes, int align_bytes, const void* input);

#define rtst_padmat_clone_mi8(rows, cols, in_lda, res_lda, mat) \
    ((rtst_int8 *) rtst_padmat_clone(rows, (cols)*sizeof(rtst_int8), \
				     (in_lda)*sizeof(rtst_int8), \
				     (res_lda)*sizeof(rtst_int8), \
				     sizeof(rtst_int8), mat))

#define rtst_padmat_clone_mi16(rows, cols, in_lda, res_lda, mat) \
    ((rtst_int16 *) rtst_padmat_clone(rows, (cols)*sizeof(rtst_int16), \
				     (in_lda)*sizeof(rtst_int16), \
				     (res_lda)*sizeof(rtst_int16), \
				      sizeof(rtst_int16), mat))

#define rtst_padmat_clone_mi32(rows, cols, in_lda, res_lda, mat) \
    ((rtst_int32 *) rtst_padmat_clone(rows, (cols)*sizeof(rtst_int32), \
				     (in_lda)*sizeof(rtst_int32), \
				     (res_lda)*sizeof(rtst_int32), \
				      sizeof(rtst_int32), mat))

#define rtst_padmat_clone_mf(rows, cols, in_lda, res_lda, mat) \
    ((float *) rtst_padmat_clone(rows, (cols)*sizeof(float), \
				 (in_lda)*sizeof(float), \
				 (res_lda)*sizeof(float), \
				 sizeof(float), mat))

#define rtst_padmat_clone_md(rows, cols, in_lda, res_lda, mat) \
    ((double *) rtst_padmat_clone(rows, (cols)*sizeof(double), \
				  (in_lda)*sizeof(double), \
				  (res_lda)*sizeof(double), \
				  sizeof(double), mat))

/* Check routines. */

extern void
_rtst_padmat_check(const void *p, const char *file, unsigned int line);

#define rtst_padmat_check_mi8(p) \
    _rtst_padmat_check((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_check_mi16(p) \
    _rtst_padmat_check((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_check_mi32(p) \
    _rtst_padmat_check((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_check_mf(p) \
    _rtst_padmat_check((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_check_md(p) \
    _rtst_padmat_check((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_check(p) \
    _rtst_padmat_check((void *)(p), __FILE__, __LINE__)


/* Delete routines. */

extern void
_rtst_padmat_del(void *p, const char *file, unsigned int line);

#define rtst_padmat_del_mi8(p) \
    _rtst_padmat_del((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_del_mi16(p) \
    _rtst_padmat_del((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_del_mi32(p) \
    _rtst_padmat_del((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_del_mf(p) \
    _rtst_padmat_del((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_del_md(p) \
    _rtst_padmat_del((void *)(p), __FILE__, __LINE__)

#define rtst_padmat_del(p) \
    _rtst_padmat_del((void *)(p), __FILE__, __LINE__)

#ifdef __cplusplus
}
#endif

#endif /* #ifndef rtst_h_INCLUDED */
