/*
  $Header: /u/drspeech/repos/rtst/rtst_padvec.c,v 1.13 1996/10/15 04:21:52 davidj Exp $

  This package provides "padvec"s, which are used to help test code.

  Padvecs are allocated like normal vectors but have extra chunks of invisible
  padding added at either end. This padding is initialized with a random
  signature, and can be checked for corruption.

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* For systems with broken header files. */
/* extern int fputs(const char* s, FILE* stream); */
/* extern int fputc(int c, FILE *stream); */
/* long nrand48(unsigned short x[3]); */
/* extern void *memcpy(void *dest, const void *src, size_t n); */

#include "rtst.h"

/* Data for one padvec. */

typedef struct
{
    unsigned char *m;           /* Pointer to allocated chunk, NULL if none. */
    size_t size;                /* Size of allocated chunk, 0 if none. */

    int allocated;              /* Whether in use. */
    size_t start;               /* Offset to start of user vector. */
    size_t length;              /* Length of user vector. */
    unsigned char sigbyte;      /* Signature byte used in padding. */
    
} PadVec;

/* Keep simple table of preallocated vectors for now. */
/* Note: Initialization relies of size of 0, not pointer being 0. */
/* TODO: Dynamically allocate table entries. */

#define PADVEC_MAX (64)

static PadVec padvec[PADVEC_MAX];

static const int PADLEN = 32;	/* Number of bytes of padding around vector. */
static const int MIN_SLOP = 64;	/* Minimum variance in start address.  */

static void
error(const char *message)
{
    fputs(message, stderr);
    fputc('\n', stderr);
    exit(1);
}

void *
rtst_padvec_new(size_t n_bytes, int align_bytes)
{
    const size_t alloc_sz = n_bytes + align_bytes + 2 * PADLEN + MIN_SLOP;
    PadVec *pv;                 /* Pointer to current padvec structure. */
    size_t align_mask;          /* Mask to get correct alignment. */
    size_t slop;                /* Amount of slop to change start address. */
    long rand_val;              /* Result of nrand48 call. */
    size_t r_offset;            /* Offset to start address. */
    unsigned char *startp;      /* Pointer to start pad section. */
    unsigned char *endp;        /* Pointer to end pad section. */
    unsigned char sig;          /* Signature byte. */
    void *retp;                 /* Pointer to return. */

    /* Scan list for non-allocated entry. If not large enough, then grow */
    /* it. This access algorithm is optimized for the case where */
    /* the same pattern of padvec sizes are repeatedly allocated and freed */
    /* in the same order.  */ 

    int i;
    for (i=0; i<PADVEC_MAX && padvec[i].allocated; i++)
        ;

    if (i==PADVEC_MAX)
        error("rtst_padvec_new(): Cannot allocate more padvecs.\n");

    /* Cache padvec structure address in pointer. */
    pv = &padvec[i];

    /* Allocate new chunk if existing block too small, or if it doesn't */
    /* exist yet.. */
    if (alloc_sz > pv->size)
    {
        if (pv->size != 0)      /* Don't free if never allocated. */
            free((void *)pv->m);

        if ((pv->m = (unsigned char *) malloc(alloc_sz)) == NULL)
            error("rtst_padvec_new(): "
                  "Malloc failed when allocating new padvec.\n");

        pv->size = alloc_sz;
    }
    pv->allocated = 1;

    /* Check that byte alignment is a power of 2. */
    if (align_bytes & (align_bytes - 1))
        error("rtst_padvec_new(): Byte alignment not a power of 2.");
    
    align_mask = ~(align_bytes - 1);

    /* Find random start place with legal alignment, and room for padding. */
    slop = MIN_SLOP + (pv->size - alloc_sz);
    rand_val = nrand48(rtst_state48);
    r_offset = (rand_val % slop) & align_mask;
    pv->start = r_offset + PADLEN;
    pv->length = n_bytes;

    /* Re-use rand val for signature byte. Use % 257 to decorrelate with */
    /* alignments. */
    sig = (unsigned char) (rand_val % 257);
    pv->sigbyte = sig;

    /* Initialize pointers to just before start and just after end of user */
    /* vector. */
    startp = pv->m + pv->start;
    retp = (void *) startp;
    endp = startp + n_bytes;    /* Byte past end of array. */
    startp--;                   /* Move to byte before start of array. */

    /* Write pad values. */
    for (i=0; i<PADLEN; i++)
    {
        *startp-- = sig;
        *endp++ = sig;
    }

    return retp;
}

void *
rtst_padvec_clone(size_t n_bytes, int align_bytes, void *vec)
{
    void *clonep = rtst_padvec_new(n_bytes, align_bytes);

    /* memcpy should be fastest way of doing this. */
    (void) memcpy(clonep, vec, n_bytes);

    return clonep;
}

void
_rtst_padvec_check(void *p, const char *file, unsigned int line)
{
    int i;
    PadVec *pv;                 /* Cache pointer to current padvec. */
    int start_bad;              /* Flag start padding corrupted. */
    int end_bad;                /* Flag end padding corrupted. */
    unsigned char *startp;      /* Start padding pointer. */
    unsigned char *endp;        /* End padding pointer. */
    unsigned char sig;          /* Signature byte. */

    /* First try to find pointer in table of padvecs. */
    for (i=0; i<PADVEC_MAX; i++)
    {
        if (padvec[i].allocated
            && p == (void *) ((unsigned char *) padvec[i].m + padvec[i].start))
            break;
    }

    if (i==PADVEC_MAX)
        error("rtst_padvec_check() : Illegal padvec pointer.");

    pv = &padvec[i];

    /* Make sure padding bytes match signature byte. */
    /* Keep two separate flags, one for start and end to tell user which */
    /* ends are corrupted. */

    start_bad = 0;
    end_bad = 0;

    startp = pv->m + pv->start;
    endp = startp + pv->length; /* Move to first byte after user array. */
    startp--;                   /* Move to first byte before user array. */

    sig = pv->sigbyte;
    
    for (i=0; i<PADLEN; i++)
    {
        /* Usually all bytes match, so optimize loop for that case by not */
        /* branching out if match fails. */
        start_bad |= (*startp-- != sig);
        end_bad |= (*endp++ != sig);
    }

    if (start_bad || end_bad)
    {
        if (!end_bad)
            rtst_failed("%s:%d: rtst_padvec_check failed, "
                        "start pad corrupted.", file, line);
        if (!start_bad)
            rtst_failed("%s:%d: rtst_padvec_check failed, "
                        "end pad corrupted.", file, line);
            
        rtst_failed("%s:%d: rtst_padvec_check failed, "
                    "start and end pad corrupted.", file, line);
    }
}

void
_rtst_padvec_del(void *p, const char *file, unsigned int line)
{
    int i;

    /* Check first. */
    _rtst_padvec_check(p, file, line);

    /* Find padvec and deallocate. */
    for (i=0; i<PADVEC_MAX; i++)
    {
        if (padvec[i].allocated
            && p == (void *) ((unsigned char *) padvec[i].m + padvec[i].start))
            break;
    }

    if (i==PADVEC_MAX)
        error("rtst_padvec_del() : "
              "Illegal padvec pointer.");

    padvec[i].allocated = 0;
}
